#include <iostream>
#include <string>

#define _WIN32_WINNT 0x0400
#include "windows.h"
/*
Copyright (C) 2004, 2005 Brian Victor Fernandes

This file is part of SuperSify.

SuperSify is a simple console based client which you can use to log into your Sify Broadband account. 
Currently, it exists only for Win32 systems.
SuperSify is a standalone client and does not require Sify's Client to work. 

SuperSify is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with XMMS mp3cue; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

SuperSify: http://broadbandforum.in/sify/supersify
Email: infernalproteus@gmail.com
*/
#include "HTTPUtils.h"
#include "XMLData.h"
#include "snmpmac.cpp"
#include "heartbeat.cpp"
#include "CmdlineParser.cpp"

string createLoginString(XMLData *xmlData);
bool isNullOrEmpty(string value);
bool firstContact(HTTPUtils *httpUtil, XMLData *data);
bool logout(HTTPUtils *httpUtil, XMLData *data);
bool loginLogout(HTTPUtils *httpUtil, XMLData *data, bool bLogout);
void CALLBACK stopBeat(unsigned long dwParam);
int endProgram(int code);

bool keyPress;

int main(int argc, char *argv[])
{
	cout << "SuperSify v0.52" << endl 
	<< "http://forum.seo4india.com/sify/supersify or " << endl
	<< "infernalproteus@gmail.com for support" << endl << endl;
	//cout << "TEST VERSION. UNSUPPORTED. DO NOT USE" << endl;

	WSADATA wsaData;
	if (WSAStartup(MAKEWORD(1, 1), &wsaData) != 0) {
		fprintf(stderr, "WSAStartup failed.\n");
		return 1;
	}

	CmdlineParser *clP = new CmdlineParser(argc, argv);
	string name = clP->getOptionValue('u');
	string password = clP->getOptionValue('p');
	if(isNullOrEmpty(name) || isNullOrEmpty(password)) {
		cout << "Usage: supersify -u username -p password [-h enable/disable] [-m macaddress] [-l] [-k]" << endl;		
		return endProgram(1);
	}	

	keyPress = !clP->getBooleanValue('k');

	
	cout << "Attempting to contact Sify server" << endl;

	HTTPUtils httpUtil;
	XMLData data;


	if(!firstContact(&httpUtil, &data))
		return endProgram(1);

	if(data.responseCode < 0)  {
		cout << "SuperSify: Server response error. Code: " << data.responseCode << endl;
		return endProgram(1);
	}


	data.userID = name;
	data.password = password;
	data.macAddress = clP->getOptionValue('m');

	bool bLogout = clP->getBooleanValue('l');
	if(!loginLogout(&httpUtil, &data, bLogout))
		return endProgram(1);
	
	

	cout << "Sify: " << data.replyMessage << endl;
	if(data.responseCode < 0) {
		cout << "SuperSify: ";
		if(bLogout)
			cout << " Logout";
		else cout << " Login";
		cout << " error. Code: " << data.responseCode << endl;
		return endProgram(1);
	}

	if(bLogout) {
		data.freeParser();
		delete clP;		
		return endProgram(0);
	}
	
	//logged in - save the session id
	FILE *temp = fopen("supersify.sid", "wt");
	if(temp) {
		fprintf(temp, "%s", data.sessionID.c_str());
		fclose(temp);				
	}
	
	//display the login info
	cout << endl << "Account Details:" << endl
		<< "Balance: " << data.accStruct.balance << endl
		<< "Product Code: " << data.accStruct.productCode << endl
		<< "Last Login: " << data.accStruct.lastLogin << endl
		<< "Expiry: " << data.accStruct.expiry << endl;

	if(data.accStruct.flag == 'y' || data.accStruct.flag == 'Y') {
		cout << "Renewal Message: " << data.accStruct.renewalMessage << endl
		<< "URL: " << data.accStruct.URL << endl << endl;
	}

	string hbString = clP->getOptionValue('h');
	bool hbForce = !strcmpi(hbString.c_str(),"enable");
	
	cout << "HeartBeat: ";
	if(data.hbStruct.hbEnable)
		cout << "Enabled";
	else cout << "Disabled";
	cout << " by Sify server. ";

	if(!(clP->getBooleanValue('h')))
		cout << "Unspecified by user.";
	else if(!hbForce)
		cout << "Disabled by user.";
	else cout << "Enabled by user.";
	
	cout << endl;

	if(hbForce || (isNullOrEmpty(hbString) && data.hbStruct.hbEnable)) {
		Heartbeat beat(&data);
		
		string bInterval = clP->getOptionValue('b');
		delete clP;

		//convert the existing heartbeat frequency value to seconds
		data.hbStruct.freq *= 60;
		if(!isNullOrEmpty(bInterval)) {
			int interval = atoi(bInterval.c_str());
			if(interval) {				
				cout << "Overriding default interval of " << data.hbStruct.freq << " seconds with " << interval << " seconds" << endl;
				data.hbStruct.freq = interval;
			}
		}
		
		cout << endl <<  "---Heartbeat commencing---" << endl;


		beat.startBeating();

		HANDLE process = GetCurrentProcess();
		SetProcessWorkingSetSize(process, -1, -1);


		while(1) {
			char option[10];
			scanf("\n%9s", option);
			if(!strcmpi(option, "quit")) {
				cout << "WARNING: Exiting without logout." << endl;
				break;
			}
			else if(!strcmpi(option, "logout")) {
				logout(&httpUtil, &data);					
				break;
			}
			else cout << "Invalid option. Allowed commands: quit, logout" << endl;			
		}
		//break the thread
		QueueUserAPC(&stopBeat, beat.bHandle, (unsigned long)&data);
		WaitForMultipleObjects(1, &beat.bHandle, true, INFINITE);
		CloseHandle(beat.bHandle);
	}
	
	data.freeParser();

	return endProgram(0);
}

bool isNullOrEmpty(string value) 
{
	return (value.length() == 0);
}

bool loginLogout(HTTPUtils *httpUtil, XMLData *data, bool bLogout) 
{
	if(bLogout) {
		FILE *temp = fopen("supersify.sid", "rt");
		if(temp) {
			fseek(temp, 0, SEEK_END);
			int size = ftell(temp);
			char *buf = new char[size];
			rewind(temp);
			fscanf(temp, "%99s", buf);
			data->sessionID = string(buf);
			cout << "Using saved session id: " << data->sessionID << endl;
			fclose(temp);			
			delete buf;
		}
		else cout << "Warning: Cannot find supersify.sid. Will attempt logout anyway." << endl;
	}
	string loginString = createLoginString(data);		
	string validateData;
	if(!bLogout)
		validateData = httpUtil->postData(loginString, data->loginURL.c_str());
	else 
		validateData = httpUtil->postData(loginString, data->logoutURL.c_str());
	
	if(!validateData.length())
		return false;

	if(!data->parseData(validateData.c_str(), validateData.length())) {
		cout << "Unable to parse validation XML" << endl;
		return false;
	}
	if(bLogout)
		unlink("supersify.sid");
	return true;
}

bool firstContact(HTTPUtils *httpUtil, XMLData *data) 
{
	string initialData = httpUtil->getData("202.144.65.70");
	if(!initialData.length())
		return false;

	if(!data->parseData(initialData.c_str(), initialData.length())) {
		cout << "Unable to parse initial XML" << endl;
		return false;
	}
	return true;
}

bool logout(HTTPUtils *httpUtil, XMLData *data) {
	cout << "Attempting to logout" << endl;
	string logoutString = createLoginString(data);
	string logoutData = httpUtil->postData(logoutString, data->logoutURL.c_str());

		
	if(!data->parseData(logoutData.c_str(), logoutData.length())) {
		cout << "SuperSify: Unable to parse logout XML" << endl;
		return false;
	}
	
	cout << "Sify: " << data->replyMessage << endl;
	if(data->responseCode < 0) {
		cout << "SuperSify: Logout error" << endl;
		return false;
	}

	unlink("supersify.sid");
	return true;
}

string createLoginString(XMLData *xmlData) {

	//get the mac address - only if it isn't overridden	
	if(isNullOrEmpty(xmlData->macAddress)) {
		MACAddr mAddr;
		mAddr.fillMacAddresses();
		string fmAddress;
		if(mAddr.mAddresses.size() > 0) {		
			string macAddress = mAddr.mAddresses[0];
			while(macAddress.length()) {
				fmAddress.append(string("-") + macAddress.substr(0,2));
				macAddress = macAddress.substr(2);
			}
		}

		else fmAddress = "crap";
		xmlData->macAddress = fmAddress.substr(1);		
	}	
	
	cout << "Using Mac Address: " << xmlData->macAddress << endl;	
	string loginString(string("srcip=") + xmlData->pvtIP);
	loginString.append(string("&username=") + xmlData->userID);
	loginString.append(string("&password=") + xmlData->password);
	loginString.append(string("&macaddress=") + xmlData->macAddress);
	loginString.append(string("&sessionid=") + xmlData->sessionID);	
	return loginString;
}


void CALLBACK stopBeat(unsigned long dwParam)  
{
	XMLData *data = (XMLData *)dwParam;
	data->hbStruct.status = "0";
}


int endProgram(int code) {
	WSACleanup();
	cout << "Thank you for using SuperSify." << endl;
	if(keyPress || code) {
		cout << "Press any key to exit." << endl;
		getch();
	}
	return code;
}
